
/*This code illustrates the bandwidth monitor functionality. It configures a single MDMA channel 21-22 to read from DMC.
 *The bandwidth monitor register is programmed to monitor a minimum bandwidth of 400 MB/s. If the throughput is less than
 *400 MB/s, the bandwidth monitor shall expire and and error message will be printed. Else, the bandwidth register can
 *be used to monitor the measured throughput instead of using the cycle count registers.
 */

#include <ccblkfn.h>
#include "adi_initialize.h"
#include "Bandwidth_Monitor_Core0.h"
#include "ddr_init.h"
#include <cdefBF609.h>
#include <defBF609.h>
#include "time.h"
#include "stdio.h"

#pragma align 32
section("L1_data_a") unsigned char ucIntDestBuff[MAX_SIZE];
section("l2_sram") unsigned char ucIntDestBuff1[MAX_SIZE];

volatile clock_t clock_start;
volatile clock_t clock_stop_mdma0;
volatile clock_t cycles_mdma0;
volatile int bwmcount_start=5120; //Min BW of 400 MB/s
volatile int bwmcount_stop;
volatile int bm_cycles;
float bm_throughput;
float throughput_mdma0;
int DMA_Done_MDMA0=0;
int BWMCNT_Expired=0;

extern void DDR2_Init(void);

void MDMA0_Interrupt_Handler(uint32_t iid, void* handlerArg);

//Macro to configure MDMA
#define MDMA_CONFIG(S_MDMA_CHANNEL,S_CFG, S_ADDRSTART, S_MOD, S_COUNT,D_MDMA_CHANNEL,D_CFG, D_ADDRSTART, D_MOD, D_COUNT)\
		*pREG_DMA##S_MDMA_CHANNEL##_CFG = S_CFG;\
		*pREG_DMA##S_MDMA_CHANNEL##_ADDRSTART = S_ADDRSTART;\
		*pREG_DMA##S_MDMA_CHANNEL##_XCNT = S_COUNT;\
		*pREG_DMA##S_MDMA_CHANNEL##_XMOD = S_MOD;\
		*pREG_DMA##D_MDMA_CHANNEL##_CFG = D_CFG;\
		*pREG_DMA##D_MDMA_CHANNEL##_ADDRSTART = D_ADDRSTART;\
		*pREG_DMA##D_MDMA_CHANNEL##_XCNT = D_COUNT;\
		*pREG_DMA##D_MDMA_CHANNEL##_XMOD = D_MOD;

//Macro to wait for the DMA done
#define WAIT_FOR_DMADONE(DMA_CHANNEL)\
		while(!(*pREG_DMA##DMA_CHANNEL##_STAT & ENUM_DMA_STAT_IRQDONE));

//Macro to disable the DMA
#define DISABLE_DMAs(S_MDMA_CHANNEL, D_MDMA_CHANNEL)\

//Uncomment this macro to add another MDMA. Else, keep it commented
//#define ADD_ANOTHER_MDMA

int main(void)
{

	//DMA buffer size = 16 K bytes
	unsigned int uiBytes=16384;

	//Initialize the DMC
	DDR2_Init();

	//Initialize the MDMA interrupt and handler here
	*pREG_SEC0_GCTL=ENUM_SEC_GCTL_EN;
	*pREG_SEC0_CCTL0=ENUM_SEC_CCTL_EN;
	adi_int_InstallHandler(INTR_MDMA0_DST, MDMA0_Interrupt_Handler, NULL, true);

	//Configure MDMA0 to read from the DDR2 bank 0
	MDMA_CONFIG(21, SOURCE_DMA_CFG,DDR2_BANK0_START_ADDRESS, SOURCE_MSIZE, uiBytes/SOURCE_MSIZE,22,DEST_DMA_CFG, ucIntDestBuff, DEST_MSIZE, uiBytes/DEST_MSIZE);

	//Initialize the bandwidht monitor register here to a minimum bandwidth of 400 MB/s (=Buffer Size*SCLK speed in MHz/Bandwidth)
	*pREG_DMA22_BWMCNT=bwmcount_start;

	//Enable the MDMA0 here
	*pREG_DMA22_CFG |= 0x1;
	*pREG_DMA21_CFG |= 0x1;

	//Add another MDMA channel here if required
	#ifdef ADD_ANOTHER_MDMA
		 MDMA_CONFIG(25, SOURCE_DMA_CFG, DDR2_BANK1_START_ADDRESS, SOURCE_MSIZE, uiBytes/SOURCE_MSIZE,26,DEST_DMA_CFG, ucIntDestBuff1, DEST_MSIZE, uiBytes/DEST_MSIZE);
		*pREG_DMA25_CFG |= 0x1;
		*pREG_DMA26_CFG |= 0x1;
	#endif

	//Start the cycle count
	clock_start = clock();

	//Wait till the MDMA0 done
	while(DMA_Done_MDMA0==0)
	{
		//Check if the bandwidth monitor count expired. If yes, print an error message and return
		if(BWMCNT_Expired)
		{
				printf("Bandwidth monitor ERROR !!! BWMCNT_CUR  expired....!!");
				return 0;
		}
	}

	//Calculate the number of cycles required for the MDMA0 to complete
	cycles_mdma0=clock_stop_mdma0-clock_start;
	bm_cycles=bwmcount_start-bwmcount_stop;

	//Disalbe the MDMAs here
	DISABLE_DMAs(21,22);
	DISABLE_DMAs(25,26);

	//Calculate and print the throughput values
	throughput_mdma0=(float)(uiBytes*CCLK_SPEED/cycles_mdma0);
	bm_throughput=(float)(uiBytes*SCLK0_SPEED/bm_cycles);
	printf("\n\n No of bytes=%d, No of core cycles measured using cycle count=%d\n\n ",uiBytes,cycles_mdma0);
	printf("Throughput measured using cycle count=%f MB/s\n\n",throughput_mdma0, bm_throughput);
	printf("\n\n No of SCLK cycles for using BW monitor=%d\n\n ", bm_cycles);
	printf("Throughput measured using bandwidth monitor=%f MB/s\n\n",throughput_mdma0, bm_throughput);

	return 0;
}


void MDMA0_Interrupt_Handler(uint32_t iid, void* handlerArg)
{
	//Stop the cycle count
	clock_stop_mdma0 = clock();

	//Check if the interrupt occurred due to bandwidth monitor expiry
	if(*pREG_DMA22_STAT&ENUM_DMA_STAT_BWMONERR)
	{
		*pREG_DMA22_STAT |= ENUM_DMA_STAT_BWMONERR;
		BWMCNT_Expired=1;
	}

	//Check if the interrupt occured due to the DMA completion
	else if(*pREG_DMA22_STAT&ENUM_DMA_STAT_IRQDONE)
	{
		*pREG_DMA22_STAT |= ENUM_DMA_STAT_IRQDONE;			 //Clear the W1C interrupt status bit
		DMA_Done_MDMA0 = 1;
		bwmcount_stop=*pREG_DMA22_BWMCNT_CUR;
	}

}
